extends Node2D

enum unit_type {player, enemy}
export(unit_type) var type

var start : Vector2
var end : Vector2
var spawner
var attack_points = 0
var defense_points = 0
var opponent
var fought = false
var current_animation = "Idle"
var current_weapon = ""
onready var animation_player = $AnimationPlayer
onready var sprite = $Sprite
var ready_to_fight = false
onready var particles = $Particles2D
func spawn(_start : Vector2, _end : Vector2, _type : String):
	if _type == "enemy":
		type = unit_type.enemy
		sprite.modulate *= Color.red * 0.95
		sprite.modulate.a = 1
		
		if rand_range(0,100) < 50:
			var key = CardsData.get_random_card_data()
			var card_data = CardsData.CARDS[key]
			change_weapon(card_data.type)
			add_stats(card_data.attack, card_data.defense)
			
		
		
		
	if type == unit_type.enemy:
		sprite.flip_h = true
	global_position = start
	start = _start
	end = _end
	move_coroutine()

var delta = 0

func play_animation(animation, weapon):
	print(animation + "-" + weapon)
	current_animation = animation
	current_weapon = weapon
	sprite.play(animation + "-" + weapon)
func move_coroutine():
	play_animation("Walk",current_weapon)
	fought = false
	ready_to_fight = false
	while delta < 1:
		global_position = start.linear_interpolate(end,delta)
		yield(get_tree(),"idle_frame")
		delta +=  get_process_delta_time()/25
		if ready_to_fight:
			return
	attack_end()
	
func prepare_to_fight():
	ready_to_fight = true

func check_opponent(opponents_node):
	if ready_to_fight and opponents_node.ready_to_fight:
		fight()
		calculate_fight()

func calculate_fight():
	var this_result = attack_points - opponent.defense_points
	var opponent_result = opponent.attack_points - defense_points
	
	print(this_result)
	print(opponent_result)
	if this_result > opponent_result:
		opponent.kill()
		wait_and_continue()
	elif this_result < opponent_result:
		kill()
		opponent.wait_and_continue()
	else:
		if rand_range(0,100) < 50:
			kill()
			opponent.wait_and_continue()
			
		else:
			opponent.kill()
			wait_and_continue()
func wait_and_continue():
	yield(sprite,"animation_finished")
	move_coroutine()
func fight():
	if fought:
		return
	fought = true
	opponent.fight()
	animation_player.play("Attack")
	play_animation("Attack",current_weapon)
	pass
func attack_end():
	animation_player.play("Attack")
	yield(animation_player,"animation_finished")
	
	var damage = 4 *  attack_points * (1 if type == unit_type.player else -1)
	GameManager.PROGRESS_BAR.set_progress(damage)
	kill()
func play_particles():
	particles.emitting = true
func kill():
	animation_player.play("Death")
func destroy():
	if rand_range(0,100) < 50:
		GameManager.DECK.add_random_broken_card()
	spawner.pop_unit(self)
	queue_free()
	pass
func _on_Area2D_area_entered(area):
	ready_to_fight = true
	opponent = area.get_parent().get_parent()
	check_opponent(opponent)
	print("area touched")
	pass # Replace with function body.

func change_weapon(weapon : String):
	play_particles()
	play_animation(current_animation,weapon)
func add_stats(attack, defense):
	attack_points += attack
	defense_points += defense
